## S1 File. R Programming Language code (text) for repeating analysis.
## Journal Ranks Analysis
## CJA Bradshaw & BW Brook
## cjabradshaw@gmail.com
## February 2016

## Remove everything
rm(list = ls())

## libraries
library(Hmisc)
library(bootstrap)

## set working directory
#setwd("~/~/") # set as appropriate

###############################################
## import journal lists
dat.ecol <- read.table("ecol.csv", header=T, sep=",") # Ecology journals data
dat.ecol.samp <- read.table("ecolsamp.csv", header=T, sep=",") # Multidisciplinary + Ecology journals sample data
dat.medi <- read.table("medi.csv", header=T, sep=",") # Medicine journals data
dat.mult <- read.table("mult.csv", header=T, sep=",") # Multidisciplinary journals data
dat.mish <- read.table("marfish.csv", header=T, sep=",") # Marine Biology & Fisheries journals data
dat.ogyn <- read.table("obsgyn.csv", header=T, sep=",") # Obstetrics & Gynaecology journals data

## Number of journals
lecol <- dim(dat.ecol)[1]
lmedi <- dim(dat.medi)[1]
lmult <- dim(dat.mult)[1]
lmish <- dim(dat.mish)[1]
logyn <- dim(dat.ogyn)[1]


#################################
## resampled/kappa-resampled rank
## jackknife rank
#################################
## Select one dataset at a time
dat <- dat.ecol
#dat <- dat.ecol.samp
#dat <- dat.medi
#dat <- dat.mult
#dat <- dat.mish
#dat <- dat.ogyn

ldat <- dim(dat)[1]
no.vec <- 1:ldat
iter <- 10000
itdiv <- iter/100
rnk.mat <- matrix(data=NA, nrow=iter, ncol=ldat)
colnames(rnk.mat) <- dat$Journal

h5ln.obs <- ldat - rank(dat$h5/log10(dat$n), ties.method="average") + 1 # h5 = Google's 5-year h-index; n = total number of papers published in year of interest (ISI)
IF13.obs <- ldat - rank(dat$IF2013, ties.method="average") + 1 # ISI's 2013 Impact Factor
IM.obs <- ldat - rank(dat$IM, ties.method="average") + 1 # ISI Immediacy Index
SNIP13.obs <- ldat - rank(dat$SNIP13, ties.method="average") + 1 # Elsevier's Source-Normalized Impact Per Paper 
SJR13.obs <- ldat - rank(dat$SJR13, ties.method="average") + 1 # Elsevier's SCImago Journal Rank 

dat.raw <- data.frame(dat$Journal, h5ln.obs, IF13.obs, IM.obs, SNIP13.obs, SJR13.obs)
avg.rnk <- apply(dat.raw[,2:6], 1, "mean")
dat.obs <- data.frame(dat$Journal, avg.rnk)

for (i in 1:iter) {
  sub.resamp <- sample(no.vec, ldat, replace=T)
  dat.resamp <- dat[sub.resamp,]
  h5ln.rnk <- ldat - rank(dat.resamp$h5/log10(dat.resamp$n), ties.method="average") + 1
  IF13.rnk <- ldat - rank(dat.resamp$IF2013, ties.method="average") + 1
  IM.rnk <- ldat - rank(dat.resamp$IM, ties.method="average") + 1
  SNIP13.rnk <- ldat - rank(dat.resamp$SNIP13, ties.method="average") + 1
  SJR13.rnk <- ldat - rank(dat.resamp$SJR13, ties.method="average") + 1
  rnk.resamp <- data.frame(dat.resamp$Journal, h5ln.rnk, IF13.rnk, IM.rnk, SNIP13.rnk, SJR13.rnk)
  colnames(rnk.resamp) <- c("journal","h5ln", "IF13", "IM", "SNIP", "SJR")
  resamp.avg.rnk <- apply(rnk.resamp[,2:6], 1, "mean")
  rnk.resamp.dat <- data.frame(rnk.resamp, resamp.avg.rnk)
  rnk.resamp.sort <- rnk.resamp.dat[order(rnk.resamp.dat[,7],decreasing=F),]
  rnk.resamp.sort.unique <- unique(rnk.resamp.sort)
  rnk.resamp.sort.unique
  
  # store resampled ranks
  lresamp <- dim(rnk.resamp.sort.unique)[1]
  sub.stor <- rep(0,lresamp)
  for (b in 1:lresamp) {
    sub.stor[b] <- which(dat$Journal == rnk.resamp.sort.unique$journal[b])
  }
  
  rnk.mat[i,sub.stor] <- rnk.resamp.sort.unique$resamp.avg.rnk
  if (i %% itdiv==0) print(i)
}
rnk.med <- apply(rnk.mat, 2, "median", na.rm=T)
rnk.lo <- apply(rnk.mat, 2, "quantile", na.rm=T, probs=0.025)
rnk.up <- apply(rnk.mat, 2, "quantile", na.rm=T, probs=0.975)

# kappa method
kappa <- 2
rnk.update <- rnk.mat
kappa.n <- 5

for (k in 1:kappa.n) {
  resamp.avg <- apply(rnk.update, 2, "mean", na.rm=T)
  resamp.sd <- apply(rnk.update, 2, "sd", na.rm=T)
  subset.mat <- rnk.new <- matrix(data=NA, nrow=iter, ncol=ldat)
  
  for (l in 1:ldat) {
    subset.sub <- which(rnk.update[,l] > (resamp.avg[l] - (kappa*resamp.sd[l])) & rnk.mat[,l] < (resamp.avg[l] + (kappa*resamp.sd[l])))
    lsubset <- length(subset.sub)
    subset.mat[1:lsubset,l] <- subset.sub
  } 
  for (l in 1:ldat) {
    rnk.new.vals <- rnk.mat[na.omit(subset.mat[,l]),l]
    lnew <- length(rnk.new.vals)
    rnk.new[1:lnew,l] <- rnk.new.vals
  }
  rnk.update <- rnk.new
  print(k)
}

rnk.upd.med <- apply(rnk.update, 2, "median", na.rm=T)
rnk.upd.lo <- apply(rnk.update, 2, "quantile", na.rm=T, probs=0.025)
rnk.upd.up <- apply(rnk.update, 2, "quantile", na.rm=T, probs=0.975)

# jackknife
jnl.se.rnk <- jnl.up.rnk <- jnl.lo.rnk <- rep(0,ldat)
theta <- function(x){mean(x)}

for (i in 1:ldat) {
  jnl.jk <- jackknife(as.numeric(dat.raw[i,2:6]), theta)
  jnl.se.rnk[i] <- jnl.jk$jack.se
  jnl.up.rnk[i] <- quantile(jnl.jk$jack.values, probs=0.975)
  jnl.lo.rnk[i] <- quantile(jnl.jk$jack.values, probs=0.025)
}

dat.out <- data.frame(dat.obs$avg.rnk,rnk.lo,rnk.med,rnk.up,rnk.upd.lo,rnk.upd.med,rnk.upd.up,jnl.lo.rnk,jnl.up.rnk,jnl.se.rnk)
colnames(dat.out) <- c("avg", "resamp.lo", "resamp.med", "resamp.up","resamp.lo.k", "resamp.med.k", "resamp.up.k","jack.lo","jack.up","jack.se")
dat.sort <- dat.out[order(dat.out[,6],decreasing=F),]
dat.sort 

## notes:  'avg' = average raw rank; 'resamp.lo', 'resamp.med', 'resamp.hi' = simple boostrap lower, median & upper 95% confidence limits
##        'resamp.lo.k', 'resamp.med.k', 'resamp.up.k' = kappa-resampled lower, median & uppper 95% confidence limits
##        'jack.lo', 'jack.up', 'jack.se' = jackknifed lower and upper 95% confidence limits, plus the jackknifed standard error (se)

#write.table(dat.sort,file="xxx.sort.csv",sep=",",dec = ".", row.names = T,col.names = TRUE) # set as appropriate

## plot error bars (top 20 journals only)
# kappa resample
errbar(rownames(dat.sort)[1:20], dat.sort$resamp.med.k[1:20], yplus=dat.sort$resamp.up.k[1:20], yminus=dat.sort$resamp.lo.k[1:20],xlab="kappa boostrapped rank", cex.axis = 0.5)

# jackknife
errbar(rownames(dat.sort)[1:20], dat.sort$avg[1:20], yplus=dat.sort$jack.up[1:20], yminus=dat.sort$jack.lo[1:20],xlab="jackknifed rank", cex.axis = 0.5)
